

#local
#code VCC_BYTE
#include "vars8.s"


; ________________________________
; get value from unsigned byte variable
;
; PEEKu8 ( &b -- n )

PEEKu8::	
	ex	hl,de		; hl -> var
	jr	ld_unsigned_e_xhl_next

; ________________________________
; get value from signed byte variable
;
; PEEKs8 ( &b -- n )

PEEKs8::	
	ex	hl,de		; hl -> var
	jr	ld_signed_e_xhl_next

; ________________________________
; store value in byte variable
;
; POKE8 ( n &b -- )

POKE8::	ex	hl,de		; hl -> var
	pop	de
	jr	ld_xhl_e_next

; ________________________________
; store value in byte variable
;
; NPOKE8 ( &b n -- )

NPOKE8::
	pop	hl		; hl -> var
	jr	ld_xhl_e_next
	


; ________________________________
; increment byte variable
;
; PPSET8 ( &b -- )

PPSET8::
	ex	hl,de		; hl->var
	inc	(hl)
	jp	pop_de_next

; ________________________________
; decrement byte variable
;
; MMSET8 ( &b -- )

MMSET8::
	ex	hl,de		; hl->var
	dec	(hl)
	jp	pop_de_next

; ________________________________
; get value from unsigned byte variable
; with post-increment
;
; PEEKPPu8 ( &b -- n )

PEEKPPu8::
	ex	de,hl		; hl->var
	ld	e,(hl)
	inc	(hl)
	jr	extend_u8_next

; ________________________________
; get value from unsigned byte variable
; with pre-decrement
;
; PEEKMMu8 ( &b -- n )

PEEKMMu8::
	ex	de,hl		; hl->var
	ld	e,(hl)
	dec	(hl)
	jr	extend_u8_next

; ________________________________
; get value from unsigned byte variable
; with pre-decrement
;
; MMPEEKu8 ( &b -- n )

MMPEEKu8::
	ex	de,hl		; hl->var
	dec	(hl)
	jr	ld_unsigned_e_xhl_next

; ________________________________
; get value from unsigned byte variable
; with post-increment
;
; PPPEEKu8 ( &b -- n )

PPPEEKu8::
	ex	de,hl		; hl->var
	inc	(hl)
	jr	ld_unsigned_e_xhl_next



; ________________________________
; get value from signed byte variable
; with post-increment
;
; PEEKPPs8 ( &b -- n )

PEEKPPs8::
	ex	de,hl		; hl->var
	ld	e,(hl)
	inc	(hl)
	jr	extend_s8_next

; ________________________________
; get value from signed byte variable
; with post-decrement
;
; PEEKMMs8 ( &b -- n )

PEEKMMs8::
	ex	de,hl		; hl->var
	ld	e,(hl)
	dec	(hl)
	jr	extend_s8_next

; ________________________________
; get value from signed byte variable
; with pre-decrement
;
; MMPEEKs8 ( &b -- n )

MMPEEKs8::
	ex	de,hl		; hl->var
	dec	(hl)
	jr	ld_signed_e_xhl_next

; ________________________________
; get value from signed byte variable
; with pre-increment
;
; PPPEEKs8 ( &b -- n )

PPPEEKs8::
	ex	de,hl		; hl->var
	inc	(hl)
	jr	ld_signed_e_xhl_next




; ================================
;	    Arithmetics
; ================================
;
; note: values on stack are always 16 bit.
; only implementations for arithmetics on variables 
; need implementations here:

; ________________________________
; add value to byte variable
;
; ADDGL8 ( n &b -- )

ADDGL8::
	ex	hl,de		; hl->var
	pop	de		; e = value

	ld	a,(hl)
	add	e
	ld	(hl),a

	jp	pop_de_next

; ________________________________
; subtract value from byte var.
;
; SUBGL8 ( n &b -- )

SUBGL8::
	ex	hl,de		; hl->var
	pop	de		; e=value

	ld	a,(hl)
	sub	e
	ld	(hl),a
	
	jp	pop_de_next

; ________________________________
; reset bits in byte variable
;
; ANDGL8 ( n &b -- )

ANDGL8::
	ex	hl,de		; hl->var
	pop	de		; e=value

	ld	a,(hl)
	and	e
	ld	(hl),a
	
	jp	pop_de_next

; ________________________________
; set bits in byte variable
;
; ORGL8 ( n &b -- )

ORGL8::	ex	hl,de		; hl->var
	pop	de		; e=value

	ld	a,(hl)
	or	e
	ld	(hl),a

	jp	pop_de_next

; ________________________________
; toggle bits in byte variable
;
; XORGL8 ( n &b -- )

XORGL8::
	ex	hl,de		; hl->var
	pop	de		; e=value

	ld	a,(hl)
	xor	e
	ld	(hl),a
	
	jp	pop_de_next

; ________________________________
; shift left byte variable
;
; SLGL8 ( n &var -- )

SLGL8::	pop	hl		; hl = shift, de->var

 	ld	a,l
 	and	7
 	jr	z,2$
 	ld	l,a		; l = shift

	ld	a,(de)		; a = var	
1$:	add	a,a
	dec	l
	jr	nz,1$
	ld	(de),a	
2$:	jp	pop_de_next

; ________________________________
; shift right unsigned byte variable
;
; SRGLu8 ( n &var -- )

SRGLu8::
	pop	hl		; hl = shift, de -> var

 	ld	a,l
 	and	7
 	jr	z,2$
 	ld	l,a		; e = shift

	ld	a,(de)		; a = var	
1$:	srl	a
	dec	l
	jr	nz,1$	
	ld	(de),a
2$:	jp	pop_de_next

; ________________________________
; shift right signed byte variable
;
; SRGLs8 ( n &var -- )

SRGLs8::
	pop	hl		; hl = shift, de -> var

 	ld	a,l
 	and	7
 	jr	z,2$
 	ld	l,a		; e = shift

	ld	a,(de)		; a = var	
1$:	sra	a
	dec	l
	jr	nz,1$	
	ld	(de),a
2$:	jp	pop_de_next



; ================================
;    Multiply, Divide, Remainder
; ================================

; ________________________________
; multiply byte variable
;
; MULGL8 ( n &b -- )

MULGL8::
	ex	hl,de		; hl->var
	pop	de		; e = value
	ld	a,(hl)		; a = var
	push	hl
	
	call	axel		; HL = L<<8 + A * DE
	ld	e,l

pop_hl_store_e_and_drop:
	pop	hl
	jp	ld_xhl_e_next

; ________________________________
; divide unsigned byte variable
;
; DIVGLu8 ( n &b -- )

DIVGLu8::
	ex	hl,de		; hl -> var
	pop	de		; de = value
	ld	a,(hl)		; a = var
	push	hl	

	ld	l,a
	ld	h,0
	call	divu		; DE = HL / DE

	jr	pop_hl_store_e_and_drop
	
; ________________________________
; divide unsigned byte variable and store remainder
;
; REMGLu8 ( n &b -- )

REMGLu8::
	ex	hl,de		; hl -> var
	pop	de		; de = value
	ld	a,(hl)		; a = var
	push	hl
	
	ld	l,a
	ld	h,0
	call	divu		; HL = HL % DE
	ex	hl,de
	
	jr	pop_hl_store_e_and_drop
	
; ________________________________
; divide signed byte variable
;
; DIVGLs8 ( n &b -- )

DIVGLs8::
	ex	hl,de		; hl -> var
	pop	de		; de = value
	push	hl

	ld	a,(hl)		; a = var	
	ld	l,a		; l = var
	add	a,a		; cy=vz
	sbc	a,a		; 0x00 / 0xff
	ld	h,a		; hl = var

	call	divs		; DE = HL / DE

	jr	pop_hl_store_e_and_drop
	
; ________________________________
; divide signed byte variable and store remainder
;
; REMGLs8 ( n &b -- )

REMGLs8::
	ex	hl,de		; hl -> var
	pop	de		; de = value
	push	hl

	ld	a,(hl)		; a = var	
	ld	l,a
	add	a,a
	sbc	a,a
	ld	h,a
	call	rems		; HL = HL % DE
	ex	hl,de
	
	jr	pop_hl_store_e_and_drop


; ________________________________
; swap two byte variables
;
; SWAPVARS8 ( &v1 &v2 -- )

SWAPVARS8::
	pop	hl
	ld	a,(de)
	ld	e,(hl)
	ld	(hl),a
	ld	a,e
	ld	(de),a
	jp	pop_de_next


; ________________________________
; classify char
;
; ISLETTER ( n -- f )

ISLETTER::
	rlc	d
	jr	nz,return_false		; n.hi != 0
ISLETTER8::
	bit	6,e
	jp	z,return_false		; bit 0x40 must be set
	set	5,e			; or 0x20: map UC -> lc
	ld	a,e
	bit	7,a
	jr	nz,1$			; $E0..$FF

; $60..$7F:	
	sub	'a'
	cp	'z' - 'a' + 1
	jp	c,return_true
	jp	return_false
	
1$:	cp	$F7			; '÷'
	jp	nz,return_true
	jp	return_false
	
; ________________________________
; classify char
;
; ISBINDIGIT ( n -- f )

ISBINDIGIT::
	rlc	d
	jr	nz,return_false		; n.hi != 0
ISBINDIGIT8::
	ld	a,e
	or	1
	cp	'1'
	jp	z,return_true
	jp	return_false
	

; ________________________________
; classify char
;
; ISDECDIGIT ( n -- f )

ISDECDIGIT::
	rlc	d
	jr	nz,return_false		; n.hi != 0
ISDECDIGIT8::
	ld	a,e
isdd:	sub	'0'
	cp	'9'-'0'+1
	jp	c,return_true
	jp	return_false

; ________________________________
; classify char
;
; ISHEXDIGIT ( n -- f )

ISHEXDIGIT::
	rlc	d
	jr	nz,return_false		; n.hi != 0
ISHEXDIGIT8::
	ld	a,e
	cp	'A'
	jr	c,isdd			; 0..9

; A-F
	or	$20			; a-f
	sub	'a'
	cp	'f'-'a'+1
	jp	c,return_true
return_false:
	jp	return_false




; ________________________________
; calculate digit value
; no error checking
; up to base 36
; returned value ≥ base => error
; returned value < base => OK
;
; DIGITVALUE ( n -- n )

DIGITVALUE::
	ld	a,e
	dec	a		; => 'A' starts at multiple of 0x20 => '@' and '`' result in ill. value
	cp	'9' + 1 - 1
	jr	c,1$		; '0'..'9'
	or	$20		; 'a'..'z'
	sub	'a' - '0' - 10 

1$:	sub	'0' - 1
	ld	e,a
	jp	extend_pos


	

#endlocal




